<?php

namespace App\Console\Commands;

use App\Core\Elastic\IndexFactory;
use Elasticsearch\Client;
use Illuminate\Console\Command;

/**
 * Class ElasticIndex
 * @package App\Console\Commands
 */
class ElasticIndex extends Command
{
    /** @var string */
    protected $signature = 'elastic:index
                                {index : name of index}
                                {action : up, down or refresh}';

    /** @var string */
    protected $description = 'Create or delete index in elasticsearch';

    /**
     * @return int
     */
    public function handle()
    {
        ini_set('memory_limit', '10G');

        $indexFactory = new IndexFactory(resolve(Client::class));
        $type = $this->argument('index');
        $action = $this->argument('action');

        $index = $indexFactory->index($type);
        if (!$index) {
            /** @var string $indexesNames Списко имен доступных индексов через запятую */
            $indexesNames = implode(', ', IndexFactory::getIndexesNamesList());
            $errorMessage = "Undefined index type '{$type}'. You probably meant one of the following: '{$indexesNames}'";

            $this->error($errorMessage);

            return 1;
        }

        switch ($action) {
            case 'up':
                if (!$index->isUp()) {
                    $index->up();
                }

                break;
            case 'down':
                if ($index->isUp()) {
                    $index->down();
                }

                break;
            case 'refresh':
                if ($index->isUp()) {
                    $index->down();
                }
                $index->up();

                break;
            default:
                $this->error("Undefined action '{$action}'. You probably meant 'up', 'down', or 'refresh'");

                return 1;
        }
        $this->info('Success');

        return 0;
    }
}
