<?php

namespace App\Console\Commands;

use App\Core\Elastic\SchedulerFactory;
use App\Core\Elastic\ScheduleRunner;
use Illuminate\Console\Command;

/**
 * Консольная команда полной переиндексации.
 *
 * Class ElasticReindex
 * @package App\Console\Commands
 */
class ElasticReindex extends Command
{
    /** @var string */
    protected $signature = 'elastic:reindex
                                    {index : name of index}
                                    {--force : run indexation immediately, not by schedule}';

    /** @var string */
    protected $description = 'Full reindex';

    /**
     * Исполняет консольную команду.
     *
     * @return int
     */
    public function handle()
    {
        ini_set('memory_limit', '10G');

        $type = $this->argument('index');
        $force = $this->option('force');

        $returnCode = $this->callSilent('elastic:index', [
            'index' => $type,
            'action' => 'refresh',
        ]);

        if ($returnCode) {
            $this->error('Error: index recreate failed');

            return $returnCode;
        }

        $this->scheduleAll($type);

        if ($force) {
            return $this->call('elastic:run');
        }

        $this->info('Success');

        return 0;
    }

    /**
     * @param string|null $type
     */
    private function scheduleAll(?string $type)
    {
        $factory = new SchedulerFactory();
        $scheduler = $factory->scheduler($type);

        $scheduler->scheduleAll(new ScheduleRunner());
    }
}
