<?php

namespace App\Core\Elastic;

use App\Core\Elastic\Category\CategoryIndex;
use App\Core\Elastic\Category\CategoryIndexer;
use App\Core\Elastic\Product\Indexing\PipelineIndexer;
use App\Core\Elastic\Product\Indexing\ProductsPipeline;
use App\Core\Elastic\Product\ProductIndex;

class IndexerFactory
{
    public function __construct(private IndexFactory $indexFactory, private ProductsPipeline $productsPipeline)
    {
    }

    public function indexer(string $type): ?Indexer
    {
        switch ($type) {
            case IndexFactory::TYPE_PRODUCT:
                /** @var ProductIndex $index */
                $index = $this->indexFactory->index($type);

                return new PipelineIndexer($this->productsPipeline, $index);
            case IndexFactory::TYPE_CATEGORY:
                /** @var CategoryIndex $index */
                $index = $this->indexFactory->index($type);

                return new CategoryIndexer($index);
            default:
                return null;
        }
    }
}
