<?php

namespace App\Core\Elastic\Product\Indexing\Actions;

use App\Core\Elastic\Product\Indexing\OfferTicket;
use Ensi\OffersClient\Api\StocksApi;
use Ensi\OffersClient\Dto\SearchStocksRequest;
use Ensi\OffersClient\Dto\Stock;
use Illuminate\Support\Collection;

class FillStocksAction
{
    public const MIN_STOCK = 4;

    public function __construct(private StocksApi $api)
    {
    }

    /**
     * @param Collection|OfferTicket[] $tickets
     */
    public function execute(Collection $tickets): void
    {
        $stocks = $this->loadStocks($tickets->keys()->all())
            ->groupBy(fn (Stock $stock) => $stock->getOfferId());

        foreach ($tickets as $ticket) {
            $target = $ticket->target;
            $stock = $stocks->get($ticket->id)?->first();

            if (!$stock) {
                $target->addStrike('Нет остатков');

                continue;
            }

            if ($stock->getStoreId()) {
                $target->storeId = $stock->getStoreId();
            } else {
                $target->addStrike('Нет склада');
            }

            $target->qty = $stock->getQty();
            if ($stock->getQty() < self::MIN_STOCK) {
                $target->addStrike('Сток товара на складе меньше ' . self::MIN_STOCK);
            }
        }
    }

    private function loadStocks(array $offerIds): Collection
    {
        $request = new SearchStocksRequest(['filter' => ['offer_id' => $offerIds]]);

        return collect($this->api->searchStocks($request)->getData());
    }
}
