<?php

namespace App\Core\Elastic\Product\Indexing\Stages;

use App\Core\Elastic\Product\Indexing\PipelineStageInterface;
use App\Core\Elastic\Product\Indexing\ProcessesTicket;
use App\Core\Elastic\Product\Indexing\ProductTicket;
use Ensi\CmsClient\Api\ProductGroupsApi;
use Ensi\CmsClient\Dto\SearchProductGroupsRequest;
use Illuminate\Support\Collection;

class ProductGroupsStage implements PipelineStageInterface
{
    use ProcessesTicket;

    public function __construct(private ProductGroupsApi $api)
    {
    }

    /**
     * @inheritDoc
     */
    public function execute(Collection $tickets): void
    {
        $productIds = $tickets->map(fn (ProductTicket $ticket) => $ticket->id)->all();
        $filter = ['products.product_id' => $productIds];
        $request = new SearchProductGroupsRequest(['filter' => $filter, 'include' => ['products']]);

        $results = $this->api->searchProductGroups($request)->getData();

        $productGroups = [];
        foreach ($results as $group) {
            foreach ($group->getProducts() as $product) {
                $productId = $product->getProductId();

                if (!isset($productGroups[$productId])) {
                    $productGroups[$productId] = [$group->getCode()];
                } else {
                    $productGroups[$productId][] = $group->getCode();
                }
            }
        }

        $this->forEachTicket(
            $tickets,
            fn (ProductTicket $ticket) => $ticket->target->groups = $productGroups[$ticket->id] ?? []
        );
    }
}
