<?php

namespace App\Domain\Products\Actions\Products;

use App\Core\Elastic\IndexFactory;
use App\Core\Elastic\ScheduleRunner;
use App\Domain\Products\Models\Product;
use Exception;
use Illuminate\Support\Collection;
use Laravie\SerializesQuery\Eloquent;
use Spatie\QueueableAction\QueueableAction;

class MarkProductsForIndexAction
{
    use QueueableAction;

    const CHUNK_SIZE = 500;

    public function __construct(private ScheduleRunner $scheduler)
    {
    }

    public function execute(array $serializedQuery): void
    {
        Eloquent::unserialize($serializedQuery)
            ->select(['id'])
            ->chunkById(self::CHUNK_SIZE, fn (Collection $chunk) => $this->handleChunk($chunk));
    }

    protected function handleChunk(Collection $chunk): void
    {
        $chunk->each(fn (Product $product) => $this->handleProduct($product));
    }

    private function handleProduct(Product $product): void
    {
        try {
            $this->scheduler->markForIndex(IndexFactory::TYPE_PRODUCT, $product->id);
        } catch (Exception $exception) {
            logger()->error(
                "Failed to put product [{$product->id}] in the indexing queue",
                ['reason' => $exception->getMessage()]
            );
        }
    }
}
