<?php

namespace App\Domain\Products\Jobs;

use App\Domain\Products\Actions\Publication\VerifyProductAction;
use App\Support\Log\ProductsLoggerAwareInterface;
use Exception;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Psr\Log\LoggerAwareTrait;

class VerifyProductJob implements ShouldQueue, ShouldBeUnique, ProductsLoggerAwareInterface
{
    use LoggerAwareTrait;
    use Dispatchable;
    use Queueable;
    use InteractsWithQueue;

    public int $uniqueFor = 3600;

    public function __construct(private readonly int $productId)
    {
    }

    public function uniqueId(): int
    {
        return $this->productId;
    }

    public function handle(VerifyProductAction $action): void
    {
        try {
            $action->execute($this->productId);
        } catch (Exception $e) {
            $this->logger?->error(
                'Error occurred while verifying product',
                ['id' => $this->productId, 'reason' => $e->getMessage()]
            );
        }
    }
}
