<?php

use App\Domain\Categories\Events\CategoryActualized;
use App\Domain\Categories\Models\Category;
use App\Domain\Classifiers\Enums\ProductStatus;
use App\Domain\Classifiers\Events\BrandActivityChanged;
use App\Domain\Classifiers\Models\Brand;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\PublishedProduct;
use function Pest\Laravel\assertDatabaseHas;
use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class)->group('publication', 'integration');

test('verify on brand activity changed', function () {
    $category = Category::factory()->actualized()->createOne();
    $product = Product::factory()
        ->inCategory($category)
        ->createOne(['status_id' => ProductStatus::AGREED]);

    Brand::query()->whereKey($product->brand_id)->update(['is_active' => false]);
    BrandActivityChanged::dispatch($product->brand_id, false);

    assertDatabaseHas(
        PublishedProduct::tableName(),
        ['id' => $product->id, 'status_id' => ProductStatus::CLOSED]
    );
});

test('verify on category actualized', function () {
    $category = Category::factory()->actualized(false)->createOne();
    $product = Product::factory()
        ->inCategory($category)
        ->createOne(['status_id' => ProductStatus::AGREED]);

    CategoryActualized::dispatch($category->id);

    assertDatabaseHas(
        PublishedProduct::tableName(),
        ['id' => $product->id, 'status_id' => ProductStatus::CLOSED]
    );
});
