<?php

namespace App\Domain\Products\Listeners;

use App\Domain\Categories\Events\CategoryActualized;
use App\Domain\Products\Jobs\VerifyProductJob;
use App\Domain\Products\Models\Product;
use App\Support\Log\ProductsLoggerAwareInterface;
use Exception;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Support\Collection;
use Psr\Log\LoggerAwareTrait;

class VerifyCategoryProductsListener implements ShouldQueue, ProductsLoggerAwareInterface
{
    use LoggerAwareTrait;

    private const CHUNK_SIZE = 500;

    public function handle(CategoryActualized $event): void
    {
        Product::whereCategoryId($event->categoryId)
            ->select(['id', 'category_id'])
            ->chunkById(
                self::CHUNK_SIZE,
                fn (Collection $products) => $products->each($this->dispatchProduct(...))
            );
    }

    private function dispatchProduct(Product $product): void
    {
        try {
            VerifyProductJob::dispatch($product->id);
        } catch (Exception $e) {
            $this->logger?->error(
                'Cannot dispatch product verification',
                ['product_id' => $product->id, 'category_id' => $product->category_id, 'reason' => $e->getMessage()]
            );
        }
    }
}
