<?php

namespace App\Http\ApiV1\Modules\Products\Queries;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\Relation;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\AllowedInclude;
use Spatie\QueryBuilder\QueryBuilder;

abstract class BaseProductsQuery extends QueryBuilder
{
    public function __construct(Builder|Relation $subject)
    {
        parent::__construct($subject);

        $this->allowedSorts(array_merge(['id', 'name', 'code'], $this->extraSorts()));
        $this->defaultSort('id');

        $this->allowedFilters(array_merge([
            AllowedFilter::exact('id'),
            AllowedFilter::exact('code'),
            AllowedFilter::partial('name'),
            AllowedFilter::scope('category_id', 'in_categories'),
            AllowedFilter::exact('brand_id'),
            AllowedFilter::exact('external_id'),
            AllowedFilter::exact('barcode'),
            AllowedFilter::exact('vendor_code'),
            AllowedFilter::exact('status_id'),
            AllowedFilter::exact('created_at'),
            AllowedFilter::exact('allow_publish'),
        ], $this->extraFilters()));

        $this->allowedIncludes(array_merge([
            'brand',
            'category',
            'images',
            AllowedInclude::custom('attributes', new PropertyDirectoryValueInclude()),
        ], $this->extraIncludes()));
    }

    protected function extraSorts(): array
    {
        return [];
    }

    protected function extraFilters(): array
    {
        return [];
    }

    protected function extraIncludes(): array
    {
        return [];
    }
}
