<?php

namespace App\Http\ApiV1\Modules\Products\Requests;

use App\Domain\Products\Models\ProductImage;
use App\Domain\Support\Models\TempFile;
use App\Http\ApiV1\Support\Requests\BaseFormRequest;
use Illuminate\Validation\Rule;

class PatchOrReplaceImagesRequest extends BaseFormRequest
{
    public function rules(): array
    {
        return array_merge(
            ['images' => ['present', 'array']],
            self::itemRules($this->route('id'))
        );
    }

    public function images(): array
    {
        return $this->validated()['images'];
    }

    public static function itemRules(int $productId): array
    {
        return [
            'images.*.name' => ['sometimes', 'nullable', 'string'],
            'images.*.sort' => ['sometimes', 'integer'],

            'images.*.id' => [
                'required_without_all:images.*.preload_file_id,images.*.url',
                'integer',
                Rule::exists(ProductImage::class, 'id')->where('product_id', $productId),
            ],

            'images.*.preload_file_id' => [
                'prohibits:images.*.id,images.*.url',
                'integer',
                Rule::exists(TempFile::class, 'id'),
            ],

            'images.*.url' => [
                'prohibits:images.*.id,images.*.preload_file_id',
                'url',
            ],
        ];
    }
}
