<?php

namespace App\Models;

use App\Domain\Products\Models\Product;
use App\Domain\Support\Models\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

/**
 * Класс-модель для сущности "Предложения мерчантов".
 *
 * Class Offer
 * @package App\Models
 * @property int $product_id - id товара
 * @property int $seller_id - id продавца
 * @property string $external_id - id товара у мерчанта
 * @property int $sale_status - статус продажи
 * @property string $storage_address - адрес хранения товара в магазине (для сборщика)
 *
 * @property-read Product $product - товар
 */
class Offer extends Model
{
    /**
     * Заполняемые поля модели
     */
    const FILLABLE = ['product_id', 'seller_id', 'external_id', 'sale_status', 'storage_address'];

    /**
     * @var array
     */
    protected $fillable = self::FILLABLE;

    /**
     * @var array
     */
    protected static $restIncludes = ['product', 'certs'];

    /**
     * @return BelongsTo
     */
    public function product(): BelongsTo
    {
        return $this->belongsTo(Product::class);
    }
}
