<?php

namespace Tests\Integration\Elastic\Products\Stages;

use App\Core\Elastic\Product\Indexing\Stages\ImagesStage;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Products\Models\ProductImageType;
use Database\Factories\Products\ProductImagesFactory;
use Ensi\LaravelEnsiFilesystem\Models\EnsiFile;

class ImageStageTest extends StageTestCase
{
    private Product $product;

    protected function setUp(): void
    {
        parent::setUp();

        $this->product = Product::factory()->createOne();
    }

    public function testMainImage(): void
    {
        $image = $this->imageFactory(ProductImageType::MAIN())->createOne();

        $result = $this->executeOne(new ImagesStage(), $this->product);

        $result->assertActive();
        $this->assertEquals(EnsiFile::public($image->file)->getUrl(), $result->catalogImage);
    }

    public function testMainImageMissing(): void
    {
        $result = $this->executeOne(new ImagesStage(), $this->product);

        $result->assertNotActive();
        $result->assertHasStrike('Нет изображения для каталога');
    }

    public function testGallery(): void
    {
        $images = $this->imageFactory(ProductImageType::GALLERY())->count(2)->create();

        $result = $this->executeOne(new ImagesStage(), $this->product);

        $this->assertEqualsCanonicalizing(
            $images->map(fn (ProductImage $image) => EnsiFile::public($image->file)->getUrl())->all(),
            $result->media
        );
    }

    public function testDescription(): void
    {
        $image = $this->imageFactory(ProductImageType::DESCRIPTION())->createOne();

        $result = $this->executeOne(new ImagesStage(), $this->product);

        $this->assertEquals($this->product->description, $result->toArray()['description']['text'] ?? null);
        $this->assertEquals(EnsiFile::public($image->file)->getUrl(), $result->toArray()['description']['image'] ?? null);
    }

    private function imageFactory(ProductImageType $type): ProductImagesFactory
    {
        return ProductImage::factory(['type' => $type->value])->owned($this->product);
    }
}
