<?php

namespace Tests\Integration\Elastic\Products\Stages;

use App\Core\Elastic\Product\Indexing\Stages\ProductGroupsStage;
use App\Domain\Products\Models\Product;
use Ensi\CmsClient\Api\ProductGroupsApi;
use Ensi\CmsClient\Dto\ProductGroup;
use Ensi\CmsClient\Dto\ProductGroupProduct;
use Ensi\CmsClient\Dto\SearchProductGroupsResponse;
use Mockery;
use Mockery\LegacyMockInterface;
use Mockery\MockInterface;

class ProductGroupStageTest extends StageTestCase
{
    private ProductGroupsApi|LegacyMockInterface|MockInterface $mockApi;

    protected function setUp(): void
    {
        parent::setUp();

        $this->mockApi = Mockery::mock(ProductGroupsApi::class);
    }

    public function testExecute(): void
    {
        $products = Product::factory()->count(2)->create();

        $apiProducts = $products->map(fn (Product $product) => new ProductGroupProduct(['product_id' => $product->id]));
        $this->mockSearchResult([
            new ProductGroup(['id' => 1, 'code' => 'group_1', 'products' => $apiProducts->all()]),
            new ProductGroup(['id' => 2, 'code' => 'group_2', 'products' => $apiProducts->all()]),
        ]);

        $results = $this->execute(new ProductGroupsStage($this->mockApi), $products->pluck('id')->all());

        foreach ($results as $result) {
            $this->assertEqualsCanonicalizing(['group_1', 'group_2'], $result->groups);
        }
    }

    public function testGroupsMissing(): void
    {
        $product = Product::factory()->createOne();
        $this->mockSearchResult([]);

        $result = $this->executeOne(new ProductGroupsStage($this->mockApi), $product);

        $this->assertEquals([], $result->groups);
    }

    private function mockSearchResult(array $groups): void
    {
        $this->mockApi->allows('searchProductGroups')
            ->andReturn(new SearchProductGroupsResponse(['data' => $groups]));
    }
}
