<?php

namespace Tests;

use App\Support\Extensions\OptionalMacros;
use Carbon\CarbonImmutable;
use Illuminate\Support\DateFactory;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Optional;
use Illuminate\Validation\Factory;
use Mockery\Adapter\Phpunit\MockeryPHPUnitIntegration;
use PHPUnit\Framework\TestCase as BaseUnitTestCase;
use Symfony\Component\VarDumper\VarDumper;
use Tests\Stubs\TranslatorStub;

class UnitTestCase extends BaseUnitTestCase
{
    use MockeryPHPUnitIntegration;

    private static bool $init = false;

    public static function setUpBeforeClass(): void
    {
        parent::setUpBeforeClass();

        self::initApplication();
    }

    public static function initApplication(): void
    {
        if (self::$init) {
            return;
        }

        VarDumper::setHandler();
        DateFactory::useClass(CarbonImmutable::class);

        Optional::mixin(new OptionalMacros());

        with(app(), function ($container) {
            $translator = new TranslatorStub();
            $validator = new Factory($translator, $container);

            $container->instance('translator', $translator);

            Validator::swap($validator);
        });

        self::$init = true;
    }
}
